<%@ page language="java" contentType="text/html; charset=UTF-8" pageEncoding="UTF-8"%>
<%@ page isELIgnored="false" %>
<%@ page errorPage="../error.jsp" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/sql" prefix="sql"%>

<sql:setDataSource var="database"
     driver="${initParam.databaseDriver}"
     url="${initParam.databaseUrl}"
     user="${initParam.databaseUsername}"
     password="${initParam.databasePassword}" />

<%-- crea il database se non esiste --%>
<sql:update dataSource="${database}">
	create database if not exists `${initParam.shopName}`
	default character set utf8
	DEFAULT COLLATE utf8_general_ci;
</sql:update>

<sql:setDataSource var="database"
     driver="${initParam.databaseDriver}"
     url="${initParam.databaseUrl}/${initParam.shopName}?useUnicode=yes&characterEncoding=UTF-8"
     user="${initParam.databaseUsername}"
     password="${initParam.databasePassword}" />

<%-- crea le tabelle nel database se non esistono --%>
<sql:update dataSource="${database}">
	create table if not exists Prodotto
	(
		ID_Prodotto		SERIAL			PRIMARY KEY,  -- ALT: AUTONUMBER (MDB), SERIAL (MySQL), COUNTER, AUTOINCREMENT (Access), IDENTITY (SQL:2003). See also http://www.w3schools.com/sql/sql_autoincrement.asp
		Nome			VARCHAR(255)	NOT NULL,
		Descrizione		VARCHAR(10000), -- ALT: CLOB, LONGVARCHAR
		Costo			NUMERIC(8,2)	NOT NULL,
		Disponibile		INT				NOT NULL,
		Immagine		VARCHAR(255),
		Immagine2		VARCHAR(255),
		Immagine3		VARCHAR(255),
		Immagine4		VARCHAR(255),
		Immagine5		VARCHAR(255),
		Video			VARCHAR(255),
		URL				VARCHAR(255),
		-- Sconto		INT 
		-- Produttore	VARCHAR(255)
		ID_Fornitore	BIGINT UNSIGNED REFERENCES Fornitore(ID_Fornitore),
		ID_Classe		BIGINT UNSIGNED REFERENCES Classe_Merceologica(ID_Classe)
		<c:choose>
			<c:when test="${initParam.databaseDriver == 'com.mysql.jdbc.Driver'}">
				<%-- MySQL --%>
			    , FULLTEXT(Nome, Descrizione) ) ENGINE=MyISAM;
			</c:when>
			<c:otherwise>
				<%-- nothing --%>
				);
			</c:otherwise>
		</c:choose>
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Cliente
	(
		ID_Cliente	SERIAL 			PRIMARY KEY,
		Nome		VARCHAR(255)	NOT NULL,
		Cognome		VARCHAR(255)	NOT NULL,
		Indirizzo	VARCHAR(255)	NOT NULL,
		email		VARCHAR(255)	NOT NULL UNIQUE,
		username	VARCHAR(255)	NOT NULL UNIQUE,
		password	VARCHAR(255)	NOT NULL,
		CHECK ((LENGTH(password) >= 5) AND
			   (username <> password)) -- MEMO: clause ignorata da MySQL -> ALTERNATIVA: TRIGGER
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Fornitore
	(
		ID_Fornitore	SERIAL			PRIMARY KEY,
		Nome			VARCHAR(255)	NOT NULL,
		Indirizzo		VARCHAR(255)	NOT NULL,
		Citta			VARCHAR(255)	NOT NULL,
		Ragione_Sociale	VARCHAR(255)
		-- Immagine		VARCHAR(255) 
		-- Video 
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Classe
	(
		ID_Classe	SERIAL			PRIMARY KEY,
		Nome 		VARCHAR(255)	NOT NULL UNIQUE,
		Descrizione	VARCHAR(1000),
		Immagine	VARCHAR(255)
		-- Video 
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Ordine
	(
		ID_Ordine	SERIAL			PRIMARY KEY,
		ID_Cliente	BIGINT UNSIGNED NOT NULL REFERENCES Cliente(ID_Cliente), -- | username?
		Stato 		VARCHAR(255)	NOT NULL,
		Data		DATE			NOT NULL,
		Totale		NUMERIC(8,2)	NOT NULL -- calcolato al momento del checkout - deve essere salvato perché il prezzo dei prodotti intanto potrebbe cambiare
		-- coupon code
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Prodotto_Ordinato
	(
		ID_Ordine	BIGINT UNSIGNED NOT NULL REFERENCES Ordine(ID_Ordine) ON DELETE CASCADE,
		ID_Prodotto	BIGINT UNSIGNED NOT NULL REFERENCES Prodotto(ID_Prodotto),
		Quantita	INT				NOT NULL,
		Costo		NUMERIC(8,2)	NOT NULL,
		PRIMARY KEY(ID_Ordine, ID_Prodotto)
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Fornitura
	(
		ID_Ordine	BIGINT UNSIGNED NOT NULL REFERENCES Ordine(ID_Ordine) ON DELETE CASCADE,
		ID_Prodotto	BIGINT UNSIGNED NOT NULL REFERENCES Prodotto(ID_Prodotto),
		Quantita	INT				NOT NULL,
		Costo		NUMERIC(8,2)	NOT NULL,
		PRIMARY KEY(ID_Ordine, ID_Prodotto)
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Carrello
	(
		ID_Cliente	BIGINT UNSIGNED NOT NULL REFERENCES Cliente(ID_Cliente) ON DELETE CASCADE, -- | username?
		ID_Prodotto	BIGINT UNSIGNED NOT NULL REFERENCES Prodotto(ID_Prodotto) ON DELETE CASCADE,
		Quantita	INT				NOT NULL,
		PRIMARY KEY(ID_Cliente, ID_Prodotto)
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists Caratteristica
	(
		ID_Caratteristica	SERIAL			PRIMARY KEY,
		Nome 				VARCHAR(255)	NOT NULL UNIQUE,
		Descrizione			VARCHAR(1000),
		Immagine			VARCHAR(255)
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists CaratterizzaProdotto
	(
		ID_Caratteristica	BIGINT UNSIGNED NOT NULL REFERENCES Caratteristica(ID_Caratteristica) ON DELETE CASCADE,
		ID_Prodotto			BIGINT UNSIGNED NOT NULL REFERENCES Prodotto(ID_Prodotto) ON DELETE CASCADE,
		Grandezza			VARCHAR(255),
		PRIMARY KEY(ID_Caratteristica, ID_Prodotto, Grandezza)
	);
</sql:update>

<sql:update dataSource="${database}">
	create table if not exists CaratterizzaClasse
	(
		ID_Caratteristica	BIGINT UNSIGNED NOT NULL REFERENCES Caratteristica(ID_Caratteristica) ON DELETE CASCADE,
		ID_Classe			BIGINT UNSIGNED NOT NULL REFERENCES Classe(ID_Classe) ON DELETE CASCADE,
		PRIMARY KEY(ID_Caratteristica, ID_Classe)
	);
</sql:update>


<%-- alla prima esecuzione registra automaticamente l'utente admin --%>
	<sql:query dataSource="${database}" var="result">
		SELECT *
		FROM Cliente
		WHERE username = 'admin'
	</sql:query>
	<c:if test="${empty result.rows}">
		<sql:update dataSource="${database}">
	   		INSERT INTO Cliente ( Nome, Cognome, Indirizzo, email, username, password )
	   		 VALUES ( 'admin', 'admin', 'admin', 'admin', 'admin', 'admin' );
		</sql:update>
		<c:set var="username" scope="session" value="admin" />
	</c:if>


<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<base href="${pageContext.request.scheme}://${pageContext.request.serverName}:${pageContext.request.serverPort}${pageContext.request.contextPath}/" />
<link href="theme/style.css" rel="stylesheet" type="text/css">
<title>${initParam.shopName} - Installazione</title>
</head>
<body>

<div class="container">
  <div class="header">
  	<%@ include file="../header.inc.jsp" %>
  <!-- end .header --></div>
  <div class="content" style="margin-left: 0; width: 100%">
  
    <h1>Installazione</h1>
    
  	Database inizializzato correttamente.<br>
  	Ricorda di cambiare frequentemente la password di "admin".
  	
  	<%-- TODO: aggiungere pulsanti import+export database --%>
  	
  <!-- end .content --></div>
  <div class="footer">
  	<%@ include file="../footer.inc.jsp" %>
  <!-- end .footer --></div>
  <!-- end .container --></div>

</body>
</html>
